/* request.cc
 * This file belongs to Worker, a filemanager for UNIX/X11.
 * Copyright (C) 2001-2004 Ralf Hoffmann.
 * You can contact me at: ralf.hoffmann@epost.de
 *   or http://www.boomerangsworld.de/worker
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* $Id: request.cc,v 1.31 2004/03/18 07:02:33 ralf Exp $ */

#include "request.h"
#include "aguix.h"
#include "awindow.h"
#include "button.h"
#include "text.h"
#include "message.h"
#include "stringgadget.h"
#include "bevelbox.h"

Requester::Requester(AGUIX *parent)
{
  aguix=parent;
  transientawin = NULL;
}

Requester::Requester( class AGUIX *parent, class AWindow *twin )
{
  aguix = parent;
  transientawin = twin;
}

Requester::~Requester()
{
}

int Requester::request(const char *title,const char *text,const char *buttons)
{
  return request( title, text, buttons, REQUEST_NONE );
}

int Requester::request(const char *title,const char *text,const char *buttons, request_flags_t flags)
{
  return request( title, text, buttons, NULL, flags );
}

int Requester::request(const char *title,const char *text,const char *buttons, AWindow *twin, request_flags_t flags)
{
  int i;
  Button **buts;
  int w, h, nbuttons;
  char **buttonss;

  nbuttons=createLines(buttons,&buttonss);
  w = h = 10;
  AWindow *win=new AWindow(aguix);
  win->create(NULL,10,10,w,h,0,title);
  if ( twin != NULL ) {
    win->setTransientForAWindow( twin );
  } else if ( transientawin != NULL ) {
    win->setTransientForAWindow( transientawin );
  } else {
    if ( aguix->getTransientWindow() != NULL ) {
      win->setTransientForAWindow( aguix->getTransientWindow() );
    }
  }
  int ty;
  ty=5;

  BevelBox *bb = (BevelBox*)win->add( new BevelBox( aguix, 5, ty, 10, 10, 1 ) );
  ty += 5;

  win->addTextFromString( text, 10, ty, 5, NULL, NULL, &ty );
  ty += 5;
  bb->resize( bb->getWidth(), ty - bb->getY() );
  ty += 5;

  win->maximizeX();
  w = win->getWidth() + 10;

  buts=(Button**)_allocsafe(nbuttons*sizeof(Button*));
  for(i=0;i<nbuttons;i++) {
    buts[i]=(Button*)win->add(new Button(aguix,0,ty,buttonss[i],1,0,i));
  }
  ty+=buts[0]->getHeight()+5;
  AGUIX::scaleElementsW( w, 5, 10, -1, false, false, (GUIElement**)buts, NULL, nbuttons );

  win->maximizeX();
  w = win->getWidth() + 10;

  for ( i = 0; i < nbuttons; i++ ) {
    buts[i]->setAcceptFocus( true );
  }
  buts[0]->takeFocus();
  win->setDoTabCycling( true );
  h=ty;
  win->resize(w,h);
  win->useStippleBackground();
  bb->resize( w - 10, bb->getHeight() );
  win->setMinSize(w,h);
  win->setMaxSize( w, h );
  win->centerScreen();
  win->show();
  AGMessage *msg;
  int endmode=-1;
  for(;endmode==-1;) {
    msg=aguix->WaitMessage(win);
    if(msg!=NULL) {
      switch(msg->type) {
        case AG_CLOSEWINDOW:
          if ( msg->closewindow.window == win->getWindow() ) {
	    if ( ( flags & REQUEST_CANCELWITHLEFT ) != 0 ) {
	      endmode = 0;
	    } else {
	      endmode = nbuttons - 1;
	    }
	  }
          break;
        case AG_BUTTONCLICKED:
	  for(i=0;i<nbuttons;i++) {
	    if(msg->button.button==buts[i]) {
	      endmode=i;
	      break;
	    }
	  }
          break;
	case AG_KEYPRESSED:
	  if ( win->isParent( msg->key.window, false ) == true ) {
	    switch(msg->key.key) {
	      case XK_KP_Enter:
	      case XK_Return:
#if 0
		endmode = 0;
		for ( i = 0; i < nbuttons; i++ ) {
		  if ( buts[i]->hasFocus() == true ) {
		    endmode = i;
		    break;
		  }
		}
#else
		if ( buts[0]->hasFocus() == true ) endmode = 0;
#endif
		break;
	      case XK_Escape:
		if ( ( flags & REQUEST_CANCELWITHLEFT ) != 0 ) {
		  endmode = 0;
		} else {
		  endmode = nbuttons - 1;
		}
		break;
	      case XK_F1:
	        endmode=0;
	        break;
	      case XK_F2:
	        endmode=1;
		if(endmode>=nbuttons) endmode = -1;
	        break;
	      case XK_F3:
	        endmode=2;
		if(endmode>=nbuttons) endmode = -1;
	        break;
	      case XK_F4:
	        endmode=3;
		if(endmode>=nbuttons) endmode = -1;
	        break;
	      case XK_F5:
	        endmode=4;
		if(endmode>=nbuttons) endmode = -1;
	        break;
	      case XK_F6:
	        endmode=5;
		if(endmode>=nbuttons) endmode = -1;
	        break;
	      case XK_F7:
	        endmode=6;
		if(endmode>=nbuttons) endmode = -1;
	        break;
	      case XK_F8:
	        endmode=7;
		if(endmode>=nbuttons) endmode = -1;
	        break;
	      case XK_F9:
	        endmode=8;
		if(endmode>=nbuttons) endmode = -1;
	        break;
	      case XK_F10:
	        endmode=9;
		if(endmode>=nbuttons) endmode = -1;
	        break;
	    }
	  }
	  break;
        case AG_SIZECHANGED:
	  for ( i = 0; i < nbuttons; i++ ) {
	    buts[i]->move( buts[i]->getX(), msg->size.newh - win->getBorderWidth() - buts[i]->getHeight() );
	  }
	  AGUIX::scaleElementsW( msg->size.neww, 5, 10, -1, false, false, (GUIElement**)buts, NULL, nbuttons );
	  break;
      }
      aguix->ReplyMessage(msg);
    }
  }
  win->close();
  delete win;
  for(i=0;i<nbuttons;i++) _freesafe(buttonss[i]);
  _freesafe(buttonss);
  _freesafe(buts);
  return endmode;
}

int Requester::string_request( const char *title, const char *text, const char *default_str, const char *buttons, char **return_str )
{
  return string_request( title, text, default_str, buttons, return_str, REQUEST_NONE );
}

int Requester::string_request( const char *title, const char *text, const char *default_str, const char *buttons, char **return_str, request_flags_t flags)
{
  return string_request( title, text, default_str, buttons, return_str, NULL, flags );
}

int Requester::string_request( const char *title, const char *text, const char *default_str, const char *buttons,char **return_str, AWindow *twin, request_flags_t flags)
{
  Button **buts;
  int w, h, nbuttons;
  char **buttonss;
  StringGadget *sg;
  int i, ty;
  
  nbuttons=createLines(buttons,&buttonss);
  w = h = 10;
  AWindow *win=new AWindow(aguix);
  win->create(NULL,10,10,w,h,0,title);
  if ( twin != NULL ) {
    win->setTransientForAWindow( twin );
  } else if ( transientawin != NULL ) {
    win->setTransientForAWindow( transientawin );
  } else {
    if ( aguix->getTransientWindow() != NULL ) {
      win->setTransientForAWindow( aguix->getTransientWindow() );
    }
  }
  ty=5;

  BevelBox *bb = (BevelBox*)win->add( new BevelBox( aguix, 5, ty, 10, 10, 1 ) );
  ty += 5;

  win->addTextFromString( text, 10, ty, 5, NULL, NULL, &ty );
  ty += 5;
  bb->resize( bb->getWidth(), ty - bb->getY() );
  ty += 5;

  sg=(StringGadget*)win->add(new StringGadget(aguix,5,ty,100,default_str,0));
  sg->setAcceptFocus( true );
  ty+=sg->getHeight()+5;
  buts=(Button**)_allocsafe(nbuttons*sizeof(Button*));
  for ( i = 0; i < nbuttons; i++ ) {
    buts[i] = (Button*)win->add( new Button( aguix, 0, ty, buttonss[i], 1, 0, i ) );
  }
  ty += buts[0]->getHeight() + 5;
  AGUIX::scaleElementsW( w, 5, 10, -1, false, false, (GUIElement**)buts, NULL, nbuttons );

  win->maximizeX();
  w = win->getWidth() + 5;

  for ( i = 0; i < nbuttons; i++ ) {
    buts[i]->setAcceptFocus( true );
  }
  h=ty;
 
  sg->setXOffset( 0 );
  if ( default_str != NULL ) {
    if ( flags == REQUEST_CURSORLEFT ) {
      sg->setCursor( 0 );
    } else if ( flags == REQUEST_CURSORRIGHT ) {
      sg->setCursor( (int)strlen( default_str ) );
    } else if ( flags == REQUEST_SELECTALL ) {
      sg->selectAll();
    }
  }

  sg->takeFocus();
  win->useStippleBackground();
  bb->resize( w - 10, bb->getHeight() );
  win->setDoTabCycling( true );
  win->resize(w,h);
  win->setMinSize(w,h);
  win->centerScreen();
  win->show();
  AGMessage *msg;
  int endmode=-1;
  for(;endmode==-1;) {
    msg=aguix->WaitMessage(win);
    if(msg!=NULL) {
      switch(msg->type) {
        case AG_CLOSEWINDOW:
          if ( msg->closewindow.window == win->getWindow() ) {
	    if ( ( flags & REQUEST_CANCELWITHLEFT ) != 0 ) {
	      endmode = 0;
	    } else {
	      endmode = nbuttons - 1;
	    }
	  }
          break;
        case AG_BUTTONCLICKED:
	  for(i=0;i<nbuttons;i++) {
	    if(msg->button.button==buts[i]) {
	      endmode=i;
	      break;
	    }
	  }
          break;
	case AG_STRINGGADGET_OK:
	  if(msg->stringgadget.sg==sg) endmode=0;
	  break;
	case AG_STRINGGADGET_CANCEL:
	  if ( msg->stringgadget.sg == sg ) {
	    if ( ( flags & REQUEST_CANCELWITHLEFT ) != 0 ) {
	      endmode = 0;
	    } else {
	      endmode = nbuttons - 1;
	    }
	  }
	  break;
	case AG_KEYPRESSED:
	  if ( win->isParent( msg->key.window, false ) == true ) {
	    // key release only accept when not first key we get
	    // means a key is pressed when we started and if we use this
	    // key it could confuse the user
	    switch(msg->key.key) {
	      case XK_KP_Enter:
	      case XK_Return:
#if 0
	        endmode=0;
		for ( i = 0; i < nbuttons; i++ ) {
		  if ( buts[i]->hasFocus() == true ) {
		    endmode = i;
		    break;
		  }
		}
#else
		if ( buts[0]->hasFocus() == true ) endmode = 0;
#endif
		break;
	      case XK_Escape:
		if ( ( flags & REQUEST_CANCELWITHLEFT ) != 0 ) {
		  endmode = 0;
		} else {
		  endmode = nbuttons - 1;
		}
		break;
	      case XK_F1:
	        endmode=0;
	        break;
	      case XK_F2:
	        endmode=1;
		if(endmode>=nbuttons) endmode = -1;
	        break;
	      case XK_F3:
	        endmode=2;
		if(endmode>=nbuttons) endmode = -1;
	        break;
	      case XK_F4:
	        endmode=3;
		if(endmode>=nbuttons) endmode = -1;
	        break;
	      case XK_F5:
	        endmode=4;
		if(endmode>=nbuttons) endmode = -1;
	        break;
	      case XK_F6:
	        endmode=5;
		if(endmode>=nbuttons) endmode = -1;
	        break;
	      case XK_F7:
	        endmode=6;
		if(endmode>=nbuttons) endmode = -1;
	        break;
	      case XK_F8:
	        endmode=7;
		if(endmode>=nbuttons) endmode = -1;
	        break;
	      case XK_F9:
	        endmode=8;
		if(endmode>=nbuttons) endmode = -1;
	        break;
	      case XK_F10:
	        endmode=9;
		if(endmode>=nbuttons) endmode = -1;
	        break;
	    }
	  }
	  break;
        case AG_SIZECHANGED:
	  for ( i = 0; i < nbuttons; i++ ) {
	    buts[i]->move( buts[i]->getX(), msg->size.newh - win->getBorderWidth() - buts[i]->getHeight() );
	  }
	  AGUIX::scaleElementsW( msg->size.neww, 5, -1, -1, true, true, (GUIElement**)(&sg), NULL, 1 );
	  AGUIX::scaleElementsW( msg->size.neww, 5, 10, -1, false, false, (GUIElement**)buts, NULL, nbuttons );
	  bb->resize( msg->size.neww - 10, bb->getHeight() );
	  break;
      }
      aguix->ReplyMessage(msg);
    }
  }
  if ( return_str != NULL ) {
    *return_str = dupstring( sg->getText() );
  }
  win->close();
  delete win;
  for(i=0;i<nbuttons;i++) _freesafe(buttonss[i]);
  _freesafe(buttonss);
  _freesafe(buts);
  return endmode;
}
