/* enterpathop.cc
 * This file belongs to Worker, a filemanager for UNIX/X11.
 * Copyright (C) 2001-2004 Ralf Hoffmann.
 * You can contact me at: ralf.hoffmann@epost.de
 *   or http://www.boomerangsworld.de/worker
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* $Id: enterpathop.cc,v 1.12 2004/03/18 07:07:04 ralf Exp $ */

#include "enterpathop.h"
#include "listermode.h"
#include "normalmode.h"
#include "worker.h"

const char *EnterPathOp::name="EnterPathOp";

EnterPathOp::EnterPathOp() : FunctionProto()
{
  enterpathside=ENTERPATHOP_THISSIDE;
  hasConfigure = true;
}

EnterPathOp::~EnterPathOp()
{
}

EnterPathOp*
EnterPathOp::duplicate() const
{
  EnterPathOp *ta=new EnterPathOp();
  ta->enterpathside=enterpathside;
  return ta;
}

bool
EnterPathOp::isName(const char *str)
{
  if(strcmp(str,name)==0) return true; else return false;
}

const char *
EnterPathOp::getName()
{
  return name;
}

int
EnterPathOp::run(ActionMessage *msg)
{
  ListerMode *lm1;
  am=msg;
  if(msg->mode!=msg->AM_MODE_DNDACTION) {
    Lister *l1 = msg->getWorker()->getActiveLister();
    if(l1!=NULL) {
      startlister=l1;
      lm1=startlister->getActiveMode();
      if(lm1!=NULL)
        if(lm1->isType("NormalMode")==true)
          normalmodeenterpath();
    }
  }
  return 0;
}

bool
EnterPathOp::save(Datei *fh)
{
  if ( fh == NULL ) return false;
  switch(enterpathside) {
    case ENTERPATHOP_OTHERSIDE:
      WConfig::configPutPair( fh, "mode", "other" );
      break;
    case ENTERPATHOP_LEFTSIDE:
      WConfig::configPutPair( fh, "mode", "left" );
      break;
    case ENTERPATHOP_RIGHTSIDE:
      WConfig::configPutPair( fh, "mode", "right" );
      break;
    default:
      WConfig::configPutPair( fh, "mode", "current" );
      break;
  }
  return true;
}

const char *
EnterPathOp::getDescription()
{
  return catalog.getLocaleCom(34);
}

void
EnterPathOp::normalmodeenterpath()
{
  NormalMode *nm1=NULL;
  ListerMode *lm1=NULL;
  Lister *ll,*lr,*tl;
  
  ll = am->getWorker()->getLister(0);
  lr = am->getWorker()->getLister(1);
  
  tl=NULL;
  switch(enterpathside) {
    case ENTERPATHOP_OTHERSIDE:
      if(startlister==ll) tl=lr;
      else tl=ll;
      break;
    case ENTERPATHOP_LEFTSIDE:
      tl=ll;
      break;
    case ENTERPATHOP_RIGHTSIDE:
      tl=lr;
      break;
    default:
      tl=startlister;
      break;
  }
  if(tl!=NULL) {
    lm1=tl->getActiveMode();
    if(lm1!=NULL) {
      if(lm1->isType("NormalMode")==true) {
        nm1=(NormalMode*)lm1;
        nm1->enterpath();
      }
    }
  }
}

int
EnterPathOp::configure()
{
  AGUIX *aguix = Worker::getAGUIX();
  Button *okb,*cb;
  AWindow *win;
  CycleButton *rcyb;
  Text *ttext;
  int tw,w,h,x,y,t1,t2;
  AGMessage *msg;
  int endmode=-1;
  char *tstr;
  
  w=h=10;
  x=y=5;
  win=new AWindow(aguix);

  tstr=(char*)_allocsafe(strlen(catalog.getLocale(293))+strlen(catalog.getLocaleCom(34))+1);
  sprintf(tstr,catalog.getLocale(293),catalog.getLocaleCom(34));
  win->create(NULL,10,10,w,h,0,tstr);
  _freesafe(tstr);

  ttext=(Text*)win->add(new Text(aguix,x,y,catalog.getLocale(323),1));
  x+=ttext->getWidth()+5;
  rcyb=(CycleButton*)win->add(new CycleButton(aguix,x,y,100,1,0,0));
  rcyb->addOption(catalog.getLocale(324));
  rcyb->addOption(catalog.getLocale(325));
  rcyb->addOption(catalog.getLocale(326));
  rcyb->addOption(catalog.getLocale(327));
  rcyb->resize(rcyb->getMaxSize(),rcyb->getHeight());
  switch(enterpathside) {
    case ENTERPATHOP_OTHERSIDE:
      rcyb->setOption(1);
      break;
    case ENTERPATHOP_LEFTSIDE:
      rcyb->setOption(2);
      break;
    case ENTERPATHOP_RIGHTSIDE:
      rcyb->setOption(3);
      break;
    default:
      rcyb->setOption(0);
      break;
  }
  
  y+=rcyb->getHeight()+5;
  tw=x+rcyb->getWidth()+5;
  if(tw>w) w=tw;
  x=5;

  t1=(strlen(catalog.getLocale(11))+2);
  t1*=aguix->getCharWidth();
  t2=(strlen(catalog.getLocale(8))+2);
  t2*=aguix->getCharWidth();
  tw=5+t1+5+t2+5;
  if(tw>w) w=tw;
  okb=(Button*)win->add(new Button(aguix,
                                   5,
                                   y,
                                   t1,
                                   catalog.getLocale(11),
                                   1,
                                   0,
                                   0));
  cb=(Button*)win->add(new Button(aguix,
                                  w-5-t2,
                                  y,
                                  t2,
                                  catalog.getLocale(8),
                                  1,
                                  0,
                                  0));
  y+=okb->getHeight()+5;
  
  h=y;
  win->setDoTabCycling( true );
  win->resize(w,h);
  win->setMaxSize(w,h);
  win->setMinSize(w,h);
  win->show();
  for(;endmode==-1;) {
    msg=aguix->WaitMessage(win);
    if(msg!=NULL) {
      switch(msg->type) {
        case AG_CLOSEWINDOW:
          if(msg->closewindow.window==win->getWindow()) endmode=1;
          break;
        case AG_BUTTONCLICKED:
          if(msg->button.button==okb) endmode=0;
          else if(msg->button.button==cb) endmode=1;
          break;
      }
      aguix->ReplyMessage(msg);
    }
  }
  
  if(endmode==0) {
    // ok
    switch(rcyb->getSelectedOption()) {
      case 1:
        enterpathside=ENTERPATHOP_OTHERSIDE;
        break;
      case 2:
        enterpathside=ENTERPATHOP_LEFTSIDE;
        break;
      case 3:
        enterpathside=ENTERPATHOP_RIGHTSIDE;
        break;
      default:
        enterpathside=ENTERPATHOP_THISSIDE;
        break;
    }
  }
  
  win->close();
  delete win;

  return endmode;
}

void EnterPathOp::setSide(enterpath_t nv)
{
  enterpathside=nv;
}

