/* informationmode.cc
 * This file belongs to Worker, a filemanager for UNIX/X11.
 * Copyright (C) 2001-2004 Ralf Hoffmann.
 * You can contact me at: ralf.hoffmann@epost.de
 *   or http://www.boomerangsworld.de/worker
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* $Id: informationmode.cc,v 1.31 2004/03/18 07:10:06 ralf Exp $ */

#include "informationmode.h"
#include "normalmode.h"
#include "worker_locale.h"
#include "wconfig.h"
#include "worker.h"
#include "partspace.h"
#include "pdatei.h"

const char *InformationMode::type="InformationMode";

InformationMode::InformationMode(Lister *parent):ListerMode(parent)
{
  x=0;
  y=0;
  w=1;
  h=1;
  lastactivefe=NULL;
  lv=NULL;
  blanked=false;
  nospaceinfo=false;
}

InformationMode::~InformationMode()
{
  lastactivefe=NULL;
}

void InformationMode::messageHandler(AGMessage *msg)
{
  int tx,ty,tw,th;
  bool ma=false;
  switch(msg->type) {
    case AG_SIZECHANGED:
      parentlister->getGeometry(&tx,&ty,&tw,&th);
      reconf(tx,ty,tw,th);
      break;
  }
  if(ma==true) parentlister->makeActive();
}

void InformationMode::on()
{
  parentlister->getGeometry(&x,&y,&w,&h);
  if ( w < 10 ) w = 10;
  if ( h < 10 ) h = 10;
  int side=parentlister->getSide();
  int m;

  lv = (FieldListView*)parentawindow->add( new FieldListView( aguix, x, y, w, h, 1 ) );
  m=(wconfig->getHBarTop(side)==true)?1:2;
  lv->setHBarState(m);
  m=(wconfig->getVBarLeft(side)==true)?1:2;
  lv->setVBarState(m);
  m=wconfig->getHBarHeight(side);
  lv->setHBarHeight(m);
  m=wconfig->getVBarWidth(side);
  lv->setVBarWidth(m);
  lv->setFont(wconfig->getFont(2+side));
  
  lv->setSize( 23 );
  for(m=0;m<23;m++) {
    lv->setText( m, 0, "" );
    lv->setPreColors( m, FieldListView::PRECOLOR_NOTSELORACT );
  }

  parentlister->setActiveMode(this);
  setName();
}

void InformationMode::off()
{
  parentawindow->remove(lv);
  delete lv;
  lv = NULL;
  parentlister->setActiveMode(NULL);
  parentlister->setName("");
  lastactivefe=NULL;
}

void InformationMode::reconf(int tx,int ty,int tw,int th)
{
  if((tw!=w)||(th!=h)||(tx!=x)||(ty!=y)) {
    w=tw;
    h=th;
    if ( w < 10 ) w = 10;
    if ( h < 10 ) h = 10;
    x=tx;
    y=ty;
    lv->resize(w,h);
    lv->move(x,y);
    update(true);
  }
}

void InformationMode::activate()
{
}

void InformationMode::deactivate()
{
}

bool InformationMode::isType(const char *str)
{
  if(strcmp(str,type)==0) return true; else return false;
}

const char *InformationMode::getType()
{
  return type;
}

const char *InformationMode::getStaticType()
{
  return type;
}


int InformationMode::configure()
{
  return 0;
}

void InformationMode::cyclicfunc(cyclicfunc_mode_t mode)
{
  update(false);
}

const char* InformationMode::getLocaleName()
{
  return getStaticLocaleName();
}

const char* InformationMode::getStaticLocaleName()
{
  return catalog.getLocale(175);
}

int InformationMode::load()
{
  return 0;
}

int InformationMode::loadBin(Datei *fh)
{
  int chunksize=fh->getInt();
  while(chunksize>0) {
    fh->getUChar();
    chunksize--;
  }
  return 0;
}

bool InformationMode::save(Datei *fh)
{
  return false;
}

void InformationMode::setName()
{
  parentlister->setName(catalog.getLocale(175));
}

void InformationMode::reconfig()
{
}

void InformationMode::update(bool force)
{
  NormalMode *nm1=NULL;
  ListerMode *lm1=NULL;
  Lister *ol=NULL;
  List *files;
  NM_specialsource *ss1;
  FileEntry *fe;
  
  ol=parentlister->getWorker()->getOtherLister(parentlister);
  if(ol!=NULL) {
    lm1=ol->getActiveMode();
    if(lm1!=NULL)
      if(lm1->isType("NormalMode")==true)
        nm1=(NormalMode*)lm1;
  }
  
  if ( ( force == true ) ||
       ( nospaceinfo == true ) )
    lastactivefe=NULL;
  
  if(nm1!=NULL) {
    files=new List();
    
    nm1->getSelFiles(files,NormalMode::NM_GETFILES_ONLYACTIVE);
    ss1=(NM_specialsource*)files->getFirstElement();
    if(ss1!=NULL) {
      fe=ss1->entry;
      if(fe!=lastactivefe) {
        lastactivefe=fe;
        showinfos();
      }
    } else {
      if ( ol->isActive() == true ) {
        // only clear LV when other side is active and has no active fe
        lastactivefe=NULL;
        showinfos();
      }
    }
    
    ss1=(NM_specialsource*)files->getFirstElement();
    while(ss1!=NULL) {
      _freesafe(ss1);
      ss1=(NM_specialsource*)files->getNextElement();
    }
    
    delete files;
  }
}

void InformationMode::showinfos()
{
  FileEntry *fe;
  int x2,len,len2;
  char buffer[1024],
       buffer2[ MAXPATHLEN + 1024 + 2 * A_BYTESFORNUMBER( int ) ],  // I need space for realpath and also other stuff
                                                                    // but this is enough
       buffer3[10],
       rights[3][4],
       *str;
  char *mstr,*kstr,*tstr;
  char *bstr1, *bstr2;
  bool showno;
  loff_t size;
  int size_size,ksize_size,msize_size;
  const char *cstr;
  size_t bufferlen;
  
  float percent;
  struct tm *timeptr;
  loff_t f1,s1;
  char numbufs[A_BYTESFORNUMBER(loff_t)], numbuff[A_BYTESFORNUMBER(loff_t)];
  time_t tvar;
  char *istr;
  
  showno=true;
  fe=lastactivefe;
  if(fe!=NULL) {
    showno=false;
    tstr=(char*)_allocsafe(strlen("%s: %s")+strlen(catalog.getLocale(176))+strlen(fe->name)+1);
    sprintf(tstr,"%s: %s",catalog.getLocale(176),fe->name);
    lv->setText( 1, 0, tstr);
    _freesafe(tstr);
    
    if((fe->isDir()==true)&&(fe->isCorrupt==false)) {
      if(fe->dirsize>=0) size=fe->dirsize;
      else size=fe->size();
    } else size=fe->size();

    size_size = NiceLongSize( size );
    str=(char*)_allocsafe(size_size+1);
    str[MakeLong2NiceStr(str,size)]='\0';

    ksize_size = NiceLongSize( ( size + 512 ) / 1024 );
    kstr = (char*)_allocsafe( ksize_size + 1 );
    kstr[MakeLong2NiceStr(kstr,(size+512)/1024)]='\0';

    msize_size = NiceLongSize( ( size + 524288 ) / 1048576 );
    mstr = (char*)_allocsafe( msize_size + 1 );
    mstr[MakeLong2NiceStr(mstr,(size+524288)/1048576)]='\0';

    bstr1 = dupstring( GetLong2Str( fe->blocks() ) );
    bstr2 = (char*)_allocsafe( strlen( catalog.getLocale( 551 ) ) +
                               strlen( bstr1 ) + 1 );
    sprintf( bstr2, catalog.getLocale( 551 ), bstr1 );
    _freesafe( bstr1 );
    
    tstr=(char*)_allocsafe(strlen("%s: %s B, %s KB, %s MB (%s)")+
                          strlen(catalog.getLocale(177))+
                          size_size+
                          ksize_size+
                          msize_size +
                          strlen( bstr2 ) + 1);
    sprintf(tstr,"%s: %s B, %s KB, %s MB (%s)",catalog.getLocale(177),str,kstr,mstr, bstr2);
    _freesafe(str);
    _freesafe(kstr);
    _freesafe(mstr);
    _freesafe( bstr2 );
    lv->setText( 2, 0, tstr);
    _freesafe(tstr);
    
    if(fe->filetype!=NULL) {
      tstr=(char*)_allocsafe(strlen("%s: %s")+
                             strlen(catalog.getLocale(178))+
                             strlen(fe->filetype->getName())+1);
      sprintf(tstr,"%s: %s",catalog.getLocale(178),fe->filetype->getName());
    } else {
      if(fe->isDir()==true) {
//        tstr=(char*)_allocsafe(strlen("%s: %s")+
//                               strlen(catalog.getLocale(178))+
//                               strlen(catalog.getLocale(109))+1);
//        sprintf(tstr,"%s: %s",catalog.getLocale(178),catalog.getLocale(109));
        tstr=(char*)_allocsafe(strlen("%s: %s")+
                               strlen(catalog.getLocale(178))+
                               strlen(wconfig->getdirtype()->getName())+1);
        sprintf(tstr,"%s: %s",catalog.getLocale(178),wconfig->getdirtype()->getName());
      } else {
        tstr=(char*)_allocsafe(strlen("%s: %s")+
                               strlen(catalog.getLocale(178))+
                               strlen(wconfig->getnotyettype()->getName())+1);
        sprintf(tstr,"%s: %s",catalog.getLocale(178),wconfig->getnotyettype()->getName());
      }
    }
    lv->setText( 3, 0, tstr);
    _freesafe(tstr);

    lv->setText( 4, 0, "" );
    if ( fe->isCorrupt == false ) {
      if ( S_ISREG( ( fe->isLink == true ) ? fe->dmode() : fe->mode() ) ) {
        enum PDatei::pdatei_handler pt;
        
        pt = PDatei::getCruncher( fe->fullname );
        switch ( pt ) {
          case PDatei::PDATEI_XPK:
            tstr = (char*)_allocsafe( strlen( catalog.getLocale( 506 ) ) +
                                      strlen( "XPK" ) + 1 );
            sprintf( tstr, catalog.getLocale( 506 ), "XPK" );
            break;
          case PDatei::PDATEI_BZIP2:
            tstr = (char*)_allocsafe( strlen( catalog.getLocale( 506 ) ) +
                                      strlen( "BZIP2" ) + 1 );
            sprintf( tstr, catalog.getLocale( 506 ), "BZIP2" );
            break;
          case PDatei::PDATEI_GZIP:
            tstr = (char*)_allocsafe( strlen( catalog.getLocale( 506 ) ) +
                                      strlen( "GZIP" ) + 1 );
            sprintf( tstr, catalog.getLocale( 506 ), "GZIP" );
            break;
          case PDatei::PDATEI_NORMAL:
            tstr = dupstring( catalog.getLocale( 507 ) );
            break;
          default:
            tstr = dupstring( "???" );
            break;
        }
        lv->setText( 4, 0, tstr );
        _freesafe( tstr );
      } 
    }

    if(fe->isLink==true) {
      str=fe->getDestination();
      
      if ( str != NULL ) {
        tstr=(char*)_allocsafe(strlen("  %s: %s")+
                               strlen(catalog.getLocale(179))+
                               strlen(str)+1);
        sprintf(tstr,"  %s: %s",catalog.getLocale(179),str);
        _freesafe(str);
        lv->setText( 5, 0, tstr);
        _freesafe(tstr);
      } else lv->setText( 5, 0, "" );
    } else lv->setText( 5, 0, "");
    
    cstr = fe->getPermissionString();
    strncpy( rights[0], cstr + 1, 3 );
    rights[0][3]='\0';
    strncpy( rights[1], cstr + 4, 3 );
    rights[1][3]='\0';
    strncpy( rights[2], cstr + 7, 3 );
    rights[2][3]='\0';
    len=strlen(catalog.getLocale(180));
    len2=strlen(catalog.getLocale(214));
    for(x2=1;x2<3;x2++) if(strlen(catalog.getLocale(214+x2))>(size_t)len2) len2=strlen(catalog.getLocale(214+x2));
    sprintf(buffer2,"%%%ds %%%ds %%s",len,len2);
    
    tstr=(char*)_allocsafe(strlen(buffer2)+
                           strlen(catalog.getLocale(180))+
                           strlen(catalog.getLocale(214))+
                           strlen(catalog.getLocale(215))+
                           strlen(catalog.getLocale(216))+
                           4+1);   // don't need so much space but prevents more allocs or max-calculation
    
    sprintf(tstr,buffer2,catalog.getLocale(180),catalog.getLocale(214),rights[0]);
    lv->setText( 7, 0, tstr);
    sprintf(tstr,buffer2,"",catalog.getLocale(215),rights[1]);
    lv->setText( 8, 0, tstr);
    sprintf(tstr,buffer2,"",catalog.getLocale(216),rights[2]);
    lv->setText( 9, 0, tstr);
    _freesafe(tstr);
    
    tstr = (char*)_allocsafe( strlen( "  %s %04o" ) +
                              strlen( catalog.getLocale( 527 ) ) +
                              A_BYTESFORNUMBER( mode_t ) + 1 );
    sprintf( tstr, "  %s %04o", catalog.getLocale( 527 ), (unsigned int)( ( fe->mode() ) & 07777 ) );
    lv->setText( 10, 0, tstr );
    _freesafe( tstr );

    str = getOwnerString( fe->userid(), fe->groupid() );
    
    tstr=(char*)_allocsafe(strlen("%s: %s")+
                           strlen(catalog.getLocale(184))+
                           strlen(str)+1);
    sprintf(tstr,"%s: %s",catalog.getLocale(184),str);
    _freesafe(str);
    lv->setText( 11, 0, tstr);
    _freesafe(tstr);

    bstr1 = dupstring( GetLong2Str( (loff_t)fe->inode() ) );
    tstr = (char*)_allocsafe( strlen( "%s: %s" ) +
                              strlen( catalog.getLocale( 548 ) ) +
                              strlen( bstr1 ) + 1 );
    sprintf( tstr, "%s: %s", catalog.getLocale( 548 ), bstr1 );
    lv->setText( 12, 0, tstr);
    _freesafe( bstr1 );
    _freesafe( tstr );

    bstr1 = dupstring( GetLong2Str( (loff_t)fe->nlink() ) );
    tstr = (char*)_allocsafe( strlen( "%s: %s" ) +
                              strlen( catalog.getLocale( 549 ) ) +
                              strlen( bstr1 ) + 1 );
    sprintf( tstr, "%s: %s", catalog.getLocale( 549 ), bstr1 );
    lv->setText( 13, 0, tstr);
    _freesafe( bstr1 );
    _freesafe( tstr );

    len = strlen( catalog.getLocale( 162 ) );
    for ( x2 = 1; x2 < 3; x2++ )
      if ( strlen( catalog.getLocale( 162 + x2 ) ) > (size_t)len ) len = strlen( catalog.getLocale( 162 + x2 ) );

    tvar = fe->lastaccess();
    timeptr = localtime( &( tvar ) );
    str = asctime( timeptr );
    str[ strlen( str ) - 1 ] = 0;
    sprintf( buffer2, "%%%ds: %%s", len );
    
    bufferlen = strlen( buffer2 ) +
                len +
                strlen( str ) + 1;
    tstr = (char*)_allocsafe( bufferlen );
    
    snprintf( tstr, bufferlen, buffer2, catalog.getLocale( 162 ), str );
    lv->setText( 15, 0, tstr );
    _freesafe( tstr );

    tvar = fe->lastmod();
    timeptr = localtime( &( tvar ) );
    str = asctime( timeptr );
    str[ strlen( str ) - 1 ] = 0;

    bufferlen = strlen( buffer2 ) +
                len +
                strlen( str ) + 1;
    tstr = (char*)_allocsafe( bufferlen );
    snprintf( tstr, bufferlen, buffer2, catalog.getLocale( 163 ), str );
    lv->setText( 16, 0, tstr );
    _freesafe( tstr );

    tvar = fe->lastchange();
    timeptr = localtime( &( tvar ) );
    str = asctime( timeptr );
    str[ strlen( str ) - 1 ] = 0;

    bufferlen = strlen( buffer2 ) +
                len +
                strlen( str ) + 1;
    tstr = (char*)_allocsafe( bufferlen );
    snprintf( tstr, bufferlen, buffer2, catalog.getLocale( 164 ), str );
    lv->setText( 17, 0, tstr );
    _freesafe( tstr );

    tstr=(char*)_allocsafe(strlen("%s: %s")+
                           strlen(catalog.getLocale(185))+
                           strlen(fe->fullname)+1);
    sprintf(tstr,"%s: %s",catalog.getLocale(185),fe->fullname);
    lv->setText( 19, 0, tstr);
    _freesafe(tstr);

    if(realpath(fe->fullname,buffer2)!=NULL) {
      tstr=(char*)_allocsafe(strlen("%s: %s")+
                             strlen(catalog.getLocale(186))+
                             strlen(buffer2)+1);
      sprintf(tstr,"%s: %s",catalog.getLocale(186),buffer2);
    } else {
      tstr=(char*)_allocsafe(strlen("%s: ???")+
                             strlen(catalog.getLocale(186))+1);
      sprintf(tstr,"%s: ???",catalog.getLocale(186));
    }
    lv->setText( 20, 0, tstr);
    _freesafe(tstr);

    if ( fe->isCorrupt == false ) {
      istr = dupstring( fe->fullname );
    } else {
      istr = ParentDir( fe->fullname, NULL );
    }
    if ( parentlister->getWorker()->PS_readSpace( istr ) == 0 ) {
      if ( parentlister->getWorker()->PS_getSpace() != 0 ) {
        percent = (double)parentlister->getWorker()->PS_getFreeSpace(); 
        percent *= 100.0;
        percent /= (double)parentlister->getWorker()->PS_getSpace();
      } else percent=0.0;

      tstr=buffer3;
      s1 = parentlister->getWorker()->PS_getSpaceH( &tstr );
      tstr=buffer2;
      f1 = parentlister->getWorker()->PS_getFreeSpaceH( &tstr );
      numbufs[WriteLong2Str( numbufs, s1 )] = '\0';
      numbuff[WriteLong2Str( numbuff, f1 )] = '\0';

      snprintf( buffer, sizeof( buffer ), "%s: %s %s (%2.0f %%) %s %s %s", catalog.getLocale( 187 ),
                numbuff,
                buffer2,
                percent,
                catalog.getLocale( 188 ),
                numbufs,
                buffer3 );
      buffer[ sizeof( buffer ) - 1 ] = '\0';
      nospaceinfo = false;
    } else {
      snprintf( buffer, sizeof( buffer ), "%s: ???", catalog.getLocale( 187 ) );
      nospaceinfo = true;
    }
    _freesafe( istr );
    lv->setText( 22, 0, buffer);
  }
  if(showno==true) {
    // fr alle anderen sind keine Informationen erhltlich
    for(int i=0;i<lv->getElements();i++) lv->setText( i, 0, "");
    lv->setText( 2, 0, catalog.getLocale(189));
  }
  lv->redraw();
}

